/**
 * @file opening.c
 *
 * Opening Name aliasing.
 *
 * @date 1998 - 2017
 * @author Richard Delorme
 * @version 4.4
 */

#include "opening.h"
#include "board.h"

#include <ctype.h>

/** opening name structure */
typedef struct OpeningName {
	const char *line; /**< move sequence */
	const char *name; /**< opening name */
} OpeningName;

/** reverse opening name */
typedef struct PositionName {
	Board board;
	const char *name;
} PositionName;

/** The list of french & english opening names */
static const OpeningName OPENING_NAME[] = {
	// French names
	{"f5d6", "perpendiculaire"},
	{"f5d6c3", "inhumaine"},
	{"f5d6c3d3c4", "tigre"},
	{"f5d6c3d3c4b3", "bill - de grey"},
	{"f5d6c3d3c4b3c5", "de grey classique"},
	{"f5d6c3d3c4b3c6", "de grey de btonnage"},
	{"f5d6c3d3c4b3d7", "de grey edmead"},
	{"f5d6c3d3c4f4c5", "tigre centrale"},
	{"f5d6c3d3c4f4c5b3", "petite tigre centrale"},
	{"f5d6c3d3c4f4c5b3c2", "triangle"},
	{"f5d6c3d3c4f4c5b3c2b4", "triangle centrale"},
	{"f5d6c3d3c4f4c5b3c2b4c6", "triangle centrale classique"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2", "triangle grosse masse"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2a3", "triangle grosse masse"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2e3", "triangle de varsovie"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2e6", "caspard"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2e6b5a5", "caspard"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2e6b5e3", "caspard retarde"},
	{"f5d6c3d3c4f4c5b3c2b4c6d2e6b5e3f3g4", "caspard de 91"},
	{"f5d6c3d3c4f4c5b3c2b4c6e6", "tamenori retarde"},
	{"f5d6c3d3c4f4c5b3c2b4c6e6b5", "tamenori retarde penloup-caspard"},
	{"f5d6c3d3c4f4c5b3c2b4c6e6e3", "tamenori retarde di meglio"},
	{"f5d6c3d3c4f4c5b3c2b4e3", "coccinelle"},
	{"f5d6c3d3c4f4c5b3c2b4e3e6c6", "coccinelle thor"},
	{"f5d6c3d3c4f4c5b3c2b4e3e6c6f6", "coccinelle moderne"},
	{"f5d6c3d3c4f4c5b3c2b4e3e6c6g4", "coccinelle classique"},
	{"f5d6c3d3c4f4c5b3c2e3", "triangle nicolet"},
	{"f5d6c3d3c4f4c5b3c2e3d2b4", "triangle nicolet centrale"},
	{"f5d6c3d3c4f4c5b3c2e3d2c6", "triangle nicolet  plat"},
	{"f5d6c3d3c4f4c5b3c2e6", "tamenori"},
	{"f5d6c3d3c4f4c5b3c2e6b4", "variante de stockholm"},
	{"f5d6c3d3c4f4c5b3c2e6b4f6c6b6", "moorlin retarde"},
	{"f5d6c3d3c4f4c5b3c2e6c6", "tamenori classique"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2a3", "tamenori japonaise"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6c1", "tamenori franaise"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6c1b6", "tamenori franaise penloup"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6c1d7", "tamenori franaise shaman"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6c7", "tamenori de sophia"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6f3", "casse-tte"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2e3a6f6", "tamenori de londres"},
	{"f5d6c3d3c4f4c5b3c2e6c6b4b5d2f7", "tamenori anglaise"},
	{"f5d6c3d3c4f4c5b3c2e6c6b6b4f6", "moorlin"},
	{"f5d6c3d3c4f4c5b3c2e6c6b6b4f6c7b5", "moorlin du centre"},
	{"f5d6c3d3c4f4c5b3c2e6c6b6b4f6g3b5", "moorlin de bton"},
	{"f5d6c3d3c4f4c5b3d2", "in my country"},
	{"f5d6c3d3c4f4c5b3d2b4", "in my country tire-bouchon"},
	{"f5d6c3d3c4f4c5b3d2c6", "in my country a plat"},
	{"f5d6c3d3c4f4c5b3d2c6b4", "in my country de lyon"},
	{"f5d6c3d3c4f4c5b3d2c6b5", "in my country alien"},
	{"f5d6c3d3c4f4c5b3d2c6b6", "in my country jensen"},
	{"f5d6c3d3c4f4c5b3d2e3", "geisha"},
	{"f5d6c3d3c4f4c5b3d2e3b4", "geisha ghirardato"},
	{"f5d6c3d3c4f4c5b3d2e3c2", "geisha pseudo-nicolet"},
	{"f5d6c3d3c4f4c5b3e2", "tigre centrale dcale"},
	{"f5d6c3d3c4f4c5b3e2c6", "dcale  plat"},
	{"f5d6c3d3c4f4c5b3e2e3", "geisha dcale"},
	{"f5d6c3d3c4f4c5b4", "grande tigre centrale"},
	{"f5d6c3d3c4f4c5b4b3", "penloup-juhem"},
	{"f5d6c3d3c4f4c5b4b3b5", "penloup-juhem requiem"},
	{"f5d6c3d3c4f4c5b4b3c2", "penloup-juhem, dfense tastet"},
	{"f5d6c3d3c4f4c5b4b3c2d7c6", "equerre centrale"},
	{"f5d6c3d3c4f4c5b4b3c2d7c7", "equerre damier"},
	{"f5d6c3d3c4f4c5b4b3c6", "penloup-juhem croix du sud"},
	{"f5d6c3d3c4f4c5b4b3e6", "penloup-juhem kaneda"},
	{"f5d6c3d3c4f4c5b4b3e6c6", "penloup-juhem andriani"},
	{"f5d6c3d3c4f4c5b4b3e6c6b5e3c2", "penloup-juhem de 92"},
	{"f5d6c3d3c4f4c5b4b3e6c6b5e3f2", "penloup-juhem ohyanagi"},
	{"f5d6c3d3c4f4c5b4b3e6c6b5e3f2a6", "penloup-juhem no-tastet"},
	{"f5d6c3d3c4f4c5b4b3e6c6b5e3f2a6a5", "penloup-juhem classique"},
	{"f5d6c3d3c4f4c5b4b3e6c6b5e3f2f7", "penloup-juhem ancienne"},
	{"f5d6c3d3c4f4c5b4b5", "tigre centrale classique"},
	{"f5d6c3d3c4f4c5b4b5c6", "tigre centrale de paris"},
	{"f5d6c3d3c4f4c5b4b5c6f3", "tigre centrale d'athnes"},
	{"f5d6c3d3c4f4c5b4b5c6f3a5", "tigre centrale grosse masse"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6", "tigre centrale murakami"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3d2", "tigre centrale japonaise"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3g6b6", "murakami tashii"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3g6f6", "murakami doyo-ka"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3g6f6g5d7g3b6", "doyo-ka polonaise"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3g6f6g5d7g3e7", "doyo-ka centrale"},
	{"f5d6c3d3c4f4c5b4b5c6f3f6", "tigre centrale plowman"},
	{"f5d6c3d3c4f4c5b4b5e6", "tigre centrale d'interversion"},
	{"f5d6c3d3c4f4c5b4c6", "tigre centrale andersson"},
	{"f5d6c3d3c4f4c5b4c6e6a3", "andersson, var.cali"},
	{"f5d6c3d3c4f4c5b4e3", "tigre centrale baltique"},
	{"f5d6c3d3c4f4e3", "brightwell"},
	{"f5d6c3d3c4f4e3f3", "brightwell intrieure"},
	{"f5d6c3d3c4f4e3f3e6", "brightwell anglaise"},
	{"f5d6c3d3c4f4e3f3e6b4", "brightwell leader"},
	{"f5d6c3d3c4f4e3f3e6b4c5", "brightwell anglaise robin"},
	{"f5d6c3d3c4f4e3f3e6b4c6", "brightwell anglaise nicolet"},
	{"f5d6c3d3c4f4e3f3e6c6", "brightwell papillon"},
	{"f5d6c3d3c4f4e3f3e6c6c5", "lepidopterix albus"},
	{"f5d6c3d3c4f4e3f3e6c6g4", "lepidopterix niger"},
	{"f5d6c3d3c4f4e3f3e6f6", "brightwell aile"},
	{"f5d6c3d3c4f4e3f3e6f6g4c5", "aile op'masse"},
	{"f5d6c3d3c4f4e3f3e6f6g4e2", "aile feldborg"},
	{"f5d6c3d3c4f4e3f3e6f6g4g3", "aile de bton"},
	{"f5d6c3d3c4f4e3f3g4", "brightwell orque"},
	{"f5d6c3d3c4f4e3f3g4e2", "orque de cambridge"},
	{"f5d6c3d3c4f4e3f3g4f6", "orque extrieure retarde"},
	{"f5d6c3d3c4f4e3f3g4g3", "orque diagonale"},
	{"f5d6c3d3c4f4e3f3g4g3f6", "orque diagonale de lille"},
	{"f5d6c3d3c4f4e3f3g4g3g5", "orque diagonale rossi"},
	{"f5d6c3d3c4f4e3f3g4g5", "orque  plat"},
	{"f5d6c3d3c4f4e3f6", "brightwell extrieure"},
	{"f5d6c3d3c4f4e3f6c6c5d7c7", "brightwell sablier"},
	{"f5d6c3d3c4f4e3f6c6c5d7e7", "brightwell d'attente"},
	{"f5d6c3d3c4f4e3f6c6c5d7e7b6", "brightwell d'attente transverse"},
	{"f5d6c3d3c4f4e3f6c6c5d7e7e6c8", "brightwell d'attente au bord"},
	{"f5d6c3d3c4f4e3f6c6c5e6", "brightwell parisienne"},
	{"f5d6c3d3c4f4e6", "tigre compacte"},
	{"f5d6c3d3c4f4e6b3", "compacte extrieure"},
	{"f5d6c3d3c4f4e6b3c2", "compacte triangle"},
	{"f5d6c3d3c4f4e6b3c2e3d2b4", "compacte triangle qvist"},
	{"f5d6c3d3c4f4e6b3c2e3d2c5", "compacte triangle de varsovie"},
	{"f5d6c3d3c4f4e6b3d2", "compacte extrieure droite"},
	{"f5d6c3d3c4f4e6b3d2c5", "variante aldaron"},
	{"f5d6c3d3c4f4e6b3d2e3", "variante des deux diagonales"},
	{"f5d6c3d3c4f4e6b3e2", "compacte extrieure diagonale"},
	{"f5d6c3d3c4f4e6b3e2c5", "variante du triplet"},
	{"f5d6c3d3c4f4e6b3e2e3", "variante du quadruplet"},
	{"f5d6c3d3c4f4e6f6", "compacte en coin"},
	{"f5d6c3d3c4f4e6f6e3", "compacte principale"},
	{"f5d6c3d3c4f4e6f6e3c5c6", "compacte d'opposition"},
	{"f5d6c3d3c4f4e6f6e3c5c6b5", "compacte d'opposition franaise"},
	{"f5d6c3d3c4f4e6f6e3c5c6b6", "compacte d'opposition sakaguchi"},
	{"f5d6c3d3c4f4e6f6e3c5g4", "compacte  plat"},
	{"f5d6c3d3c4f4e6f6e7", "compacte de grey"},
	{"f5d6c3d3c4f4f6", "tigre diagonale"},
	{"f5d6c3d3c4f4f6b4", "kung"},
	{"f5d6c3d3c4f4f6b4c2", "kung diagonale"},
	{"f5d6c3d3c4f4f6b4c2e2", "kung diagonale, variante 42"},
	{"f5d6c3d3c4f4f6b4c2f3", "kung diagonale, variante 33"},
	{"f5d6c3d3c4f4f6b4f3", "kung plate"},
	{"f5d6c3d3c4f4f6b4f3e6e3g5b5", "kung du pion isol"},
	{"f5d6c3d3c4f4f6b4f3e6e3g5g6", "kung bton"},
	{"f5d6c3d3c4f4f6f3", "tigre diagonale moderne"},
	{"f5d6c3d3c4f4f6f3e3", "tigre diagonale franaise"},
	{"f5d6c3d3c4f4f6f3e3b4", "suite franaise"},
	{"f5d6c3d3c4f4f6f3e3g5g4", "popov (interversion)"},
	{"f5d6c3d3c4f4f6f3e3g5g6", "japon 93 (interversion)"},
	{"f5d6c3d3c4f4f6f3e6", "tigre scorpion"},
	{"f5d6c3d3c4f4f6f3e6e7c6", "scorpion piau-puget"},
	{"f5d6c3d3c4f4f6f3e6e7c6g6f8", "scorpion piau de bton"},
	{"f5d6c3d3c4f4f6f3e6e7c6g6f8c5", "scorpion piau-puget, var. de bton"},
	{"f5d6c3d3c4f4f6f3e6e7c6g6g5", "scorpion piau centrale"},
	{"f5d6c3d3c4f4f6f3e6e7d7", "nouveau scorpion"},
	{"f5d6c3d3c4f4f6f3e6e7d7c5b6", "scorpion excentr"},
	{"f5d6c3d3c4f4f6f3e6e7d7c5c6", "scorpion group"},
	{"f5d6c3d3c4f4f6f3e6e7d7c5c6g6d8", "scorpion jacpoth retard"},
	{"f5d6c3d3c4f4f6f3e6e7d7c5c6g6g5", "scorpion wada (interversion)"},
	{"f5d6c3d3c4f4f6f3e6e7d7g6d8", "scorpion jacpoth"},
	{"f5d6c3d3c4f4f6f3e6e7d7g6f8", "scorpion fausse piau"},
	{"f5d6c3d3c4f4f6f3e6e7d7g6g5", "scorpion wada"},
	{"f5d6c3d3c4f4f6f3e6e7f7", "scorpion classique"},
	{"f5d6c3d3c4f4f6f3e6e7f7c5b6b4", "scorpion des 2 pions isols"},
	{"f5d6c3d3c4f4f6f3e6e7f7c5b6b5", "scorpion de varsovie"},
	{"f5d6c3d3c4f4f6f3e6e7f7c5b6g5", "scorpion du luxembourg"},
	{"f5d6c3d3c4f4f6f3e6e7f7c5b6g6", "scorpion du pion isol"},
	{"f5d6c3d3c4f4f6f3g4", "tigre diagonale damier"},
	{"f5d6c3d3c4f4f6f3g4g3c5", "tigre diagonale ralle"},
	{"f5d6c3d3c4f4f6f3g4g3e6b3", "bicorne"},
	{"f5d6c3d3c4f4f6f3g4g5", "popov (interversion)"},
	{"f5d6c3d3c4f4f6g5", "tigre diagonale classique"},
	{"f5d6c3d3c4f4f6g5c6", "tigre diagonale holgersson"},
	{"f5d6c3d3c4f4f6g5e3", "brightwell retarde"},
	{"f5d6c3d3c4f4f6g5e3f3g4", "popov"},
	{"f5d6c3d3c4f4f6g5e3f3g4e2", "popov brisson"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3", "popov au bord"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3c6", "popov sharalle"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3c7", "popov le saout"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3e6", "popov murakami"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3g3", "popov murakami retarde"},
	{"f5d6c3d3c4f4f6g5e3f3g6", "tigre japon 93"},
	{"f5d6c3d3c4f4f6g5e3f3g6e2", "suite de bton"},
	{"f5d6c3d3c4f4f6g5e3f3g6f7", "tau"},
	{"f5d6c3d3c4f4f6g5e6", "tigre diagonale groupe"},
	{"f5d6c3d3c4f4f6g5e6c5", "groupe anglaise "},
	{"f5d6c3d3c4f4f6g5e6d7", "tigre diagonale milanaise"},
	{"f5d6c3d3c4f4f6g5e6f7", "groupe japonaise"},
	{"f5d6c3d3c4f4f6g5e6f7c7", "japonaise corbeau"},
	{"f5d6c3d3c4f4f6g5e6f7d7", "japonaise juhem-caspard"},
	{"f5d6c3d3c4f4f6g5e6f7d7c5", "japonaise juhem-caspard"},
	{"f5d6c3d3c4f4f6g5e6f7d7c5g3e3", "japonaise suekuni"},
	{"f5d6c3d3c4f4f6g5e6f7e3", "japonaise suekuni"},
	{"f5d6c3d3c4f4f6g5e6f7g6", "japonaise de la roussette"},
	{"f5d6c3d3c4f4f6g5e6f7h5", "japonaise douteuse"},
	{"f5d6c3d3c4f4f6g5f3", "tigre diagonale hashimoto"},
	{"f5d6c3d3c4f4f6g5g6", "tigre diagonale tom-pouce"},
	{"f5d6c3d3c5", "parker"},
	{"f5d6c3d3c5f4e3", "parker groupe"},
	{"f5d6c3d3c5f4f3", "parker rectangulaire"},
	{"f5d6c3f4", "inconnue"},
	{"f5d6c3g5", "ganglion"},
	{"f5d6c4", "italienne"},
	{"f5d6c4d3c5", "chat"},
	{"f5d6c4d3c5b4", "nicolet"},
	{"f5d6c4d3c5b4c3d2b6c6b5e6a3a5", "nicolet historique"},
	{"f5d6c4d3c5b4d7f4e3f3", "nicolet moderne"},
	{"f5d6c4d3c5f4d7", "tanida inverse"},
	{"f5d6c4d3c5f4e3f3c2", "romaine"},
	{"f5d6c4d3c5f4e3f3c2b4", "chat en damier"},
	{"f5d6c4d3c5f4e3f3c2b4b5d2d7c7e7f6", "chat marconi"},
	{"f5d6c4d3c5f4e3f3c2c6", "romaine connexe"},
	{"f5d6c4d3c5f4e3f3c2c6d7", "romaine handel"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5c3b4c1f2a3e2a5", "chat penloup"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5c3b4c1f2a4", "chat glisse au bord"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5e7", "chat andriani"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5g3c7a5", "chat russe"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5g3c7a5f2", "romaine des minis diagos"},
	{"f5d6c4d3c5f4e3f3c2c6e6d2g4b6b5g3d1c3", "chat svirskiy"},
	{"f5d6c4d3c5f4e3f3g4", "chat amricaine"},
	{"f5d6c4d3e6", "italienne"},
	{"f5d6c4g5", "ganglion"},
	{"f5d6c5", "perpendiculaire classique"},
	{"f5d6c5f4d3", "cheval"},
	{"f5d6c5f4d3c4", "cheval centrale"},
	{"f5d6c5f4d3c4c3", "cheval crochet"},
	{"f5d6c5f4d3c4c3c6e3", "crochet en coin"},
	{"f5d6c5f4d3c4c3c6e3b5", "crochet tastet"},
	{"f5d6c5f4d3c4c3c6e3f6", "crochet pseudo-rose"},
	{"f5d6c5f4d3c4c3c6e6b5", "crochet intrieur"},
	{"f5d6c5f4d3c4e3", "cheval muller"},
	{"f5d6c5f4d3c4e3c3", "muller d't symtrique"},
	{"f5d6c5f4d3c4e3e6", "muller en coins"},
	{"f5d6c5f4d3c4e3e6f6g5c3e7", "muller fuse"},
	{"f5d6c5f4d3c4e3e6f6g5c3g6", "muller en coins dcale"},
	{"f5d6c5f4d3e3", "cheval transversale"},
	{"f5d6c5f4d3e3g4g3", "cheval daunas"},
	{"f5d6c5f4d3e3g4g3e2c4e6", "daunas aberration optique (interversion)"},
	{"f5d6c5f4d3e3g4g3e2f6e6", "daunas kyu syu"},
	{"f5d6c5f4d3e3g4g3e6c4", "daunas aberration optique"},
	{"f5d6c5f4d3e3g4g3e6c4e2", "aberration optique droite"},
	{"f5d6c5f4d3e3g4g3e6c4f2", "aberration optique diagonale"},
	{"f5d6c5f4d3e3g4g5", "cheval roques"},
	{"f5d6c5f4d3e3g4g5e2c4f3", "roques argentine"},
	{"f5d6c5f4d3e3g4g5e6c4", "roques penloup antisymtrique"},
	{"f5d6c5f4d3e3g4g5e6f6", "roques caspard"},
	{"f5d6c5f4d7", "beta"},
	{"f5d6c5f4e3c4", "ball"},
	{"f5d6c5f4e3c6d3f3", "ralle"},
	{"f5d6c5f4e3c6d3f6", "rose"},
	{"f5d6c5f4e3c6d3f6e6d7e7", "rose greenberg"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7b6", "greenberg alard (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3", "greenberg classique"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3b6", "greenberg alard"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8", "greenberg polyakov"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8e2b6", "suite polyakov"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8e2b6e8", "greenberg polyakov penloup"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8e8", "greenberg polyakov de barcelone"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8e8f8f7g8e2b5c2", "greenberg polyakov grosse masse"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8b8e8f8f7g8e2b5c3", "greenberg polyakov shastein"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f3d8c8f7", "greenberg originelle"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f8", "dfense tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f8b6", "dfense tamenori du double accs"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f8d8", "rfutation tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7c4f8g5", "dfense tamenori finlandaise"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7d8", "greenberg classique (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7d8c8", "greenberg semi-originelle"},
	{"f5d6c5f4e3c6d3f6e6d7e7c7d8c8f7f3", "greenberg originelle (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7e7f3", "greenberg classique (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7e7f3c4c7b6", "greenberg alard (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7e7f3d8", "greenberg wahlberg"},
	{"f5d6c5f4e3c6d3f6e6d7g3", "rose diagonale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4", "rose transverse"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3", "transverse principale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b5", "transverse du pion isol"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b5a4", "transverse go"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b5a5", "transverse droite au bord"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6", "transverse trois croix"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6a3", "transverse trois croix de l'arche"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6b5", "transverse trois croix mercedes"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6c3b5", "transverse trois croix classique"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6c3b5a6", "transverse trois croix de bton"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3b6c3b5a6g5a5a4a3f7", "rose bton (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3g5", "transverse pineuse"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3g5c3b5a5a4a3b6a6f7", "rose bton (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4b3g5c3b5e2", "brightstein retarde (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4c3", "transverse compacte"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4e2", "brightstein acclre"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4e7", "rose piau"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4e7c3g4", "rose piau classique"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4e7c3g5", "rose piau-marconi"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4", "rose transverse retarde"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6a6c2", "rose brisson"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6a6d8", "rose tardy"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6a6f7", "rose bton"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6a6f7e2", "rose bton colle"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6a6f7f2", "rose bton d'extraction"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5a5a4a3b6c7", "rose fautive"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5e2", "brightstein retarde"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5e2d2", "dfense bhagat"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4b3b5e2f3", "dfense danoise"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4d2e2", "rose collay"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2", "rose brightstein"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b3", "brightstein andersson"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b3d2", "brightstein andersson morillon"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b3g6f3d2", "brightstein andersson centrale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b3g6f3h5", "brightstein andersson au bord"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5", "brightstein classique"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5d2c7b3", "brightstein lazard"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5d2c7b6c2", "brightstein torben"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5d2c7b6e8", "brightstein piau"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5d2c7g6", "brightstein piau (interversion)"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5d2f7", "brightstein clef anglaise"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5g6", "brightstein tastet"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5g6c7", "brightstein tastet 2 contre 1"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2b5g6f2", "brightstein tastet ko"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2d2", "brightstein bhagat"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2f3", "brightstein cric"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2f3d2", "brightstein cric centrale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2f3g6", "brightstein cric diagonale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7", "rose tournante"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7d2e7f2c8", "tournante tourniquet"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7d2e7f2e1", "tournante paradoxale"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7d2e7f2e1f3c8g4", "paradoxale russie"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7d2e7f2e1f3c8h3", "paradoxale de sacrifice"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7d2e7f2e2", "tournante mange"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3f7g6", "tournante tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g6", "rose daunas"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g6c3", "rose daunas tournante"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g6g5", "rose daunas microb"},
	{"f5d6c5f4e3c6d3f6e6d7g4", "rose plate"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4", "rose plate transverse"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4b3", "plate transverse principale"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4b3b6a3", "plate transverse de l'arche"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4b3b6b5", "plate transverse mercedes"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4b3b6c3", "plate transverse classique"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4e2", "rose stenvenson"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4g3", "rose amadeus"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4g3g5", "amadeus du hbleur"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4b4g3g6", "amadeus mercedes"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5", "rose plate tournante (rotating flat)"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1", "plate tournante ouverte (?)"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2b5", "plate tournante de tchliabinsk"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2b5g3f3h4", "tchliabinsk droite au bord"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2b5g3f3h6", "tchliabinsk diagonale au bord"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2f2", "plate tournante japonaise"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2f2f1", "plate tournante plbenne"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2f2g6", "plate tournante kaneda"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2c1e2f2h3", "plate tournante, rfutation tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7", "plate tournante ferme (?)"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2c8f3c7d8e2", "johnson"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2c8f3c7d8e8g3e2f1", "feinder"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2c8f3c7d8e8g3e2h4", "jotel"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2e2f1c8f3c1", "plate tournante ferme, var. ouverte"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2e2f1c8f3c7", "plate tournante ferme, var. ferme"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2e2f3c1f1", "plate tournante ferme, var. ouverte"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2e2f3c1g3", "cygne"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2e2f3c1h3", "semi-tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6", "rose murakami"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6c3", "murakami tournante"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6g5e7c3", "murakami occidentale"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6g5e7f8", "murakami japonaise"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6h6", "murakami bhagat"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3", "rose bruyninckx"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3e7", "greenberg retarde"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g5h4", "bruyninckx d'encerclement"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6f3", "bruyninckx classique"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6g5", "bruyninckx mercedes"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6g5e7", "bruyninckx mercedes nordique"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6g5h5", "bruyninckx mercedes koala"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6h3", "bruyninckx arche"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6h3e7", "bruyninckx arche ralle"},
	{"f5d6c5f4e3c6d3f6e6d7g4g3g6h3g5", "bruyninckx arche tortue"},
	{"f5d6c5f4e3c6d3g5", "coup 8"},
	{"f5d6c5f4e3c6d7", "bhagat"},
	{"f5d6c5f4e3c6d7e6", "bhagat, var.principale"},
	{"f5d6c5f4e3c6e6", "inoue"},
	{"f5d6c5f4e3c6f3", "shaman"},
	{"f5d6c5f4e3c6f3c3d3c4", "shaman triangle"},
	{"f5d6c5f4e3c6f3c3d3c4b3", "shaman triangle mur"},
	{"f5d6c5f4e3c6f3c3d3c4b4", "shaman triangle transverse"},
	{"f5d6c5f4e3c6f3d3", "shaman tau"},
	{"f5d6c5f4e3c6f3d3c4b4", "tau don quichotte"},
	{"f5d6c5f4e3c6f3d3c4e2", "tau cerf"},
	{"f5d6c5f4e3c6f3g5", "shaman plate"},
	{"f5d6c5f4e3c6f3g5e6f6d7b5", "shaman plate ancienne"},
	{"f5d6c5f4e3c6f3g5e6f6d7d3", "shaman plate moderne"},
	{"f5d6c5f4e3c6f3g6", "shaman diagonale"},
	{"f5d6c5f4e3c6f3g6e6", "shaman diagonale jensen"},
	{"f5d6c5f4e3c6f3g6f6", "shaman diagonale vallund"},
	{"f5d6c5f4e3d3", "mimura"},
	{"f5d6c6", "collay"},
	{"f5f4", "parallle"},
	{"f5f6", "diagonale"},
	{"f5f6e6f4", "diagonale"},
	{"f5f6e6f4c3", "ishii"},
	{"f5f6e6f4c3c4", "ishii transverse"},
	{"f5f6e6f4c3c4d3", "ishii tigre"},
	{"f5f6e6f4c3c4g5", "ishii heath"},
	{"f5f6e6f4c3d6", "ishii colle"},
	{"f5f6e6f4c3d6f3c4", "ishii cannibale"},
	{"f5f6e6f4c3d6f3c4c5b4", "cannibale cannibalesque"},
	{"f5f6e6f4c3d6f3c4c5b4a5", "cannibale 11 au bord"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b3", "cannibale diagonale"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b3e3", "cannibale diagonale maruoka"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b3g5", "cannibale diagonale transverse"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b5", "cannibale classique"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b5a6b6g4", "cannibale tezuka"},
	{"f5f6e6f4c3d6f3c4c5b4a5a3c6b5a6e3", "cannibale murakami"},
	{"f5f6e6f4c3d6f3c4c5b4c6", "cannibale centrale"},
	{"f5f6e6f4c3d6f3c4c5b6c6b5e7", "zbre"},
	{"f5f6e6f4c3d6f3c4c5b6g4", "charriot"},
	{"f5f6e6f4c3d6f3c4c5b6g4g3", "troka"},
	{"f5f6e6f4c3d6f3c4c5b6g4h3", "charriot au bord"},
	{"f5f6e6f4c3d6f3c4c5f2", "cannibale p"},
	{"f5f6e6f4c3d6f3e3", "ishii piau"},
	{"f5f6e6f4c3d6f3e3d3c5", "ishii piau droite"},
	{"f5f6e6f4c3d6f3e3d3c6", "ishii piau diagonale"},
	{"f5f6e6f4c3d7", "ishii dcolle"},
	{"f5f6e6f4c3d7e3", "ishii lazard"},
	{"f5f6e6f4c3d7e3d6e7c4", "ishii t noir"},
	{"f5f6e6f4c3d7e3d6e7c5c4", "ishii matignon"},
	{"f5f6e6f4c3d7e3d6e7c5c6", "ishii elysenne"},
	{"f5f6e6f4c3d7f3", "ishii dcolle en coin"},
	{"f5f6e6f4c3d7f3d6g5e3", "ishii des danseurs"},
	{"f5f6e6f4c3d7f3d6g5g4e3e2", "panthon"},
	{"f5f6e6f4c3d7f3d6g5g4e3g3", "ishii ancienne"},
	{"f5f6e6f4c3d7f3d6g5g4e3g3g6", "ishii ancienne de la mini-diagonale"},
	{"f5f6e6f4c3d7f3d6g5g4e3g3h4", "ishii ancienne droite au bord"},
	{"f5f6e6f4c3d7f3d6g5g4e3g6", "ishii dcolle classique"},
	{"f5f6e6f4c3d7f3d6g5g4e3g6e7", "ishii de sacrifice"},
	{"f5f6e6f4c3d7f3d6g5g4e3g6e7g3f7c4", "ishii, var. d'vitement acclre"},
	{"f5f6e6f4c3d7f3d6g5g4e3g6e7g3f7f8", "ishii, var. d'vitement"},
	{"f5f6e6f4c3d7f3d6g5g4e3g6h6", "ishii bton"},
	{"f5f6e6f4c3d7f3d6g5g4e3h5", "ishii nordique"},
	{"f5f6e6f4c3d7f3d6g5g4e3h5e7", "ishii nordique vallund du pion blanc"},
	{"f5f6e6f4c3d7f3d6g5g4e3h5g6", "ishii nordique vallund"},
	{"f5f6e6f4c3d7f3d6g5g4g6", "ishii penloup"},
	{"f5f6e6f4c3e7", "ishii droite"},
	{"f5f6e6f4c3e7f3e3d3e2g5c5", "ishii t blanc"},
	{"f5f6e6f4d3", "citadine"},
	{"f5f6e6f4e3", "diagonale classique"},
	{"f5f6e6f4e3c5c4d3", "maruoka"},
	{"f5f6e6f4e3c5c4e7", "tanida"},
	{"f5f6e6f4e3c5c4e7b5e2f2", "tanida diagonale"},
	{"f5f6e6f4e3c5c4e7b5e2f3", "tanida droite"},
	{"f5f6e6f4e3c5c4e7g4", "avion"},
	{"f5f6e6f4e3c5c4e7g4f3", "avion droit"},
	{"f5f6e6f4e3c5c4e7g4g3", "avion diagonal"},
	{"f5f6e6f4e3c5g5", "cambridge"},
	{"f5f6e6f4e3d6", "chemine"},
	{"f5f6e6f4f3", "raton laveur"},
	{"f5f6e6f4f3c5f7", "raton shaman"},
	{"f5f6e6f4g3", "coquelicot"},
	{"f5f6e6f4g3f3", "hamilton"},
	{"f5f6e6f4g3g4", "sprague"},
	{"f5f6e6f4g4", "peccerillo"},
	{"f5f6e6f4g5", "heath"},
	{"f5f6e6f4g5d6", "heath chemine"},
	{"f5f6e6f4g5d6e7", "heath chemine diagonale"},
	{"f5f6e6f4g5d6e7g4f3h5e3", "heath chemine diagonale centrale"},
	{"f5f6e6f4g5d6e7g4f3h5h6h7", "heath chemine diagonale bton"},
	{"f5f6e6f4g5d6e7g6c5c4", "heath chemine russe"},
	{"f5f6e6f4g5d6f7", "heath chemine droite"},
	{"f5f6e6f4g5e7d7", "heath drumel"},
	{"f5f6e6f4g5e7e3", "heath comp'oth (bat)"},
	{"f5f6e6f4g5e7e3d6", "heath comp'oth groupe"},
	{"f5f6e6f4g5e7e3d6d3g4g6f3", "heath comp'oth groupe moderne"},
	{"f5f6e6f4g5e7e3d6d3g4g6h5", "heath comp'oth groupe, var. de bton"},
	{"f5f6e6f4g5e7e3f3", "heath comp'oth droite"},
	{"f5f6e6f4g5e7e3f3d3c3", "heath comp'oth cannibale"},
	{"f5f6e6f4g5e7e3f3d3c5", "heath comp'oth sauvage"},
	{"f5f6e6f4g5e7e3f3d3e2", "heath comp'oth cano"},
	{"f5f6e6f4g5e7e3g4", "heath comp'oth centrale"},
	{"f5f6e6f4g5e7e3g4d3", "heath comp'oth gnrale"},
	{"f5f6e6f4g5e7e3g4d3c5", "heath comp'oth plate"},
	{"f5f6e6f4g5e7e3g4d3c5g6f3", "heath comp'oth plate, var. d'insertion"},
	{"f5f6e6f4g5e7e3g4d3c5g6h5h6h7h4h3", "heath comp'oth plate, var. de bton"},
	{"f5f6e6f4g5e7e3g4d3c6", "heath comp'oth prtendante"},
	{"f5f6e6f4g5e7e3g4d3c6d6", "heath comp'oth ralle"},
	{"f5f6e6f4g5e7e3g4d3c6g6f3", "heath comp'oth prtendante, var. d'insertion"},
	{"f5f6e6f4g5e7e3g4d3c6g6h5",  "heath comp'oth prtendante, var. de bton"},
	{"f5f6e6f4g5e7e3g4f3", "heath comp'oth buridan"},
	{"f5f6e6f4g5e7e3g6",  "heath comp'oth diagonale"},
	{"f5f6e6f4g5e7e3g6d3", "heath comp'oth murakami"},
	{"f5f6e6f4g5e7e3g6d3c5d6c4", "heath comp'oth murakami au centre"},
	{"f5f6e6f4g5e7e3g6d3c5d6f3g4h4", "heath comp'oth murakami au bord"},
	{"f5f6e6f4g5e7e3g6d3f3g4h4", "heath comp'oth murakami au bord"},
	{"f5f6e6f4g5e7e3g6d6c4", "heath comp'oth orthodoxe"},
	{"f5f6e6f4g5e7e3g6d6c4c5b5", "heath comp'oth damier"},
	{"f5f6e6f4g5e7e3g6d6c4d3", "heath comp'oth brightwell"},
	{"f5f6e6f4g5e7e3g6d6c4f7",  "heath comp'oth informatique"},
	{"f5f6e6f4g5e7e3g6d6f3f7", "heath comp'oth takizawa"},
	{"f5f6e6f4g5e7e3g6f3", "heath comp'oth chinoise"},
	{"f5f6e6f4g5e7f7", "heath classique"},
	{"f5f6e6f4g5e7f7c5f3g3h3h5", "heath mirage"},
	{"f5f6e6f4g5e7f7c5f3g4", "heath mystere"},
	{"f5f6e6f4g5e7f7g6", "heath marconi"},
	{"f5f6e6f4g5e7f7h5", "heath au bord"},
	{"f5f6e6f4g5e7f7h5d3", "heath au bord extrieure"},
	{"f5f6e6f4g5e7f7h5d3c6", "heath ancienne"},
	{"f5f6e6f4g5e7f7h5d3c6d7", "heath bracchi"},
	{"f5f6e6f4g5e7f7h5d3c6d7c4g4h4c3", "heath en coin"},
	{"f5f6e6f4g5e7f7h5d3c6d7c4g4h4d6", "heath japonaise"},
	{"f5f6e6f4g5e7f7h5d3c6d7c4g4h4e3", "heath standard"},
	{"f5f6e6f4g5e7f7h5e3", "heath au bord mdiane"},
	{"f5f6e6f4g5e7f7h5e3d3f3", "heath shaman"},
	{"f5f6e6f4g5e7f7h5e3d3g4g6f3", "heath shaman retarde"},
	{"f5f6e6f4g5e7f7h5e3d3g4h4f3", "heath carr blanc"},
	{"f5f6e6f4g5e7f7h5e3d3g4h4g3", "heath scorpion"},
	{"f5f6e6f4g5e7f7h5g4", "heath au bord diagonale"},
	{"f5f6e6f4g5e7f7h5g4g6", "heath diagonale sur le prbord"},
	{"f5f6e6f4g5e7f7h5g4g6e3d3f3", "heath shaman retarde"},
	{"f5f6e6f4g5e7f7h5g4g6e3f3", "heath ka"},
	{"f5f6e6f4g5e7f7h5g4g6e3f8", "heath variante 56"},
	{"f5f6e6f4g5e7f7h5g4g6e3h4", "heath kawazoe"},
	{"f5f6e6f4g5e7f7h5g4h4", "heath diagonale au bord"},
	{"f5f6e6f4g5g6", "heath d'vitement"},
	{"f5f6e6f4g5g6g4e7f3", "heath d'vitement malgache"},
	{"f5f6e6f4g6", "campagnarde"},
	{"f5f6e6f4g6c5", "campagnarde droite"},
	{"f5f6e6f4g6c5g4g5e3", "campagnarde berner"},
	{"f5f6e6f4g6c5g4g5f3e3c4", "campagnarde damier"},
	{"f5f6e6f4g6c5g4g5f3e3h5", "campagnarde classique ou 11 au bord"},
	{"f5f6e6f4g6c5g4g5h4", "campagnarde 9 au bord"},
	{"f5f6e6f4g6c5g4g5h4h3f3g3e3", "campagnarde milanaise"},
	{"f5f6e6f4g6c5g4g5h4h3f3g3h5h6e3", "campagnarde ???"},
	{"f5f6e6f4g6c5g4g5h4h3f3h5", "campagnarde clou"},
	{"f5f6e6f4g6c5g4g5h4h6e3", "campagnarde danoise"},
	{"f5f6e6f4g6c5g4g5h4h6f3", "campagnarde ???"},
	{"f5f6e6f4g6c6","campagnarde dcolle"},
	{"f5f6e6f4g6c6c5", "campagnarde nordique"},
	{"f5f6e6f4g6c6g5", "campagnarde traineau"},
	{"f5f6e6f4g6d6", "campagnarde colle"},
	{"f5f6e6f4g6d6e3d3g4f3g3", "campagnarde tanigushi"},
	{"f5f6e6f4g6d6d3g5", "campagnarde kitajima"},
	{"f5f6e6f4g6d6e3", "campagnarde transverse"},
	{"f5f6e6f4g6d6e3d3g4f3g3", "campagnarde taniguchi"},
	{"f5f6e6f4g6d6e3d3g5", "campagnarde kitajima"},
	{"f5f6e6f4g6d6e3f3g4g5d3e2c5", "campagnarde ralle"},
	{"f5f6e6f4g6d6e3f3g4g5f2", "campagnarde shibata"},
	{"f5f6e6f4g6d6g4f7", "campagnarde cannibale"},
	{"f5f6e6f4g7", "ouverture x"},

	// English names
	{"f5d6", "perpendicular opening"},
	{"f5d6c3d3c4", "tiger"},
	{"f5d6c3d3c4b3", "aubrey, tanaka"},
	{"f5d6c3d3c4f4c5b3c2", "rose-bill"},
	{"f5d6c3d3c4f4c5b3c2e6", "tamenori"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3", "ishii"},
	{"f5d6c3d3c4f4c5b4b5c6f3e6e3g6f6g5d7g3", "mainline tiger"},
	{"f5d6c3d3c4f4e3", "brightwell"},
	{"f5d6c3d3c4f4e6", "leader's tiger"},
	{"f5d6c3d3c4f4f6", "stephenson"},
	{"f5d6c3d3c4f4f6b4", "kung"},
	{"f5d6c3d3c4f4f6f3", "comp'oth"},
	{"f5d6c3d3c4f4f6f3e6e7d7g6f8f7g5h6", "f.a.t. draw"},
	{"f5d6c3d3c4f4f6g5", "no-kung"},
	{"f5d6c3d3c4f4f6g5e3f3g4h3g3f2", "no-kung (continuation)"},
	{"f5d6c3g5", "ganglion"},
	{"f5d6c4d3c5", "cat"},
	{"f5d6c4d3c5f4e3f3c2b4b3", "berner"},
	{"f5d6c4d3c5f4e3f3c2c6", "sakaguchi"},
	{"f5d6c4d3e6", "italian"},
	{"f5d6c4g5", "no-cat"},
	{"f5d6c4g5c6", "swallow"},
	{"f5d6c4g5c6c5d7d3b4c3e3f3", "no-cat (continuation)"},
	{"f5d6c5f4d3", "horse"},
	{"f5d6c5f4e3c6d3f3", "ralle"},
	{"f5d6c5f4e3c6d3f6e6d7", "rose"},
	{"f5d6c5f4e3c6d3f6e6d7e7", "greenberg"},
	{"f5d6c5f4e3c6d3f6e6d7e7", "dawg"}, 
	{"f5d6c5f4e3c6d3f6e6d7g3c4", "rose-birth"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4", "rose-birdie"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4",  "rose-tamenori"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4b4c3", "rose-tamenori-kling"},
	{"f5d6c5f4e3c6d3f6e6d7g3c4g5c3b4e2", "brightstein"},
	{"f5d6c5f4e3c6d3f6e6d7g4", "flat"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4", "rotating flat"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g5c3f7d2e7f2", "rotating flat (kling continuation)"},
	{"f5d6c5f4e3c6d3f6e6d7g4c4g6", "murakami variation"},
	{"f5d6c5f4e3c6d7", "bhagat"},
	{"f5d6c5f4e3c6e6", "inoue"},
	{"f5d6c5f4e3c6e6f6", "iago"},
	{"f5d6c5f4e3c6f3", "shaman"},
	{"f5d6c5f4e3c6f3", "danish"},
	{"f5d6c5f4e3d3", "mimura"},
 	{"f5f4", "parallel opening"},
	{"f5f6", "diagonal opening"},
	{"f5f6c4f4", "semi-wing variation"},
	{"f5f6d3f4", "wing variation"},
	{"f5f6e6d6c3", "buffalo"},
	{"f5f6e6d6c3",  "kenichi variation"},
	{"f5f6e6d6c3d3", "hokuriku buffalo"},
	{"f5f6e6d6c3f4c6d3e3d2", "tanida buffalo"},
	{"f5f6e6d6c3g4c6", "maruoka buffalo"},
	{"f5f6e6d6c5", "cow"},
	{"f5f6e6d6c5e3d3", "rose-v-toth"},
	{"f5f6e6d6c5e3d3c4c3", "landau"},
	{"f5f6e6d6c5e3d3c4c6b5", "maruoka"},
	{"f5f6e6d6c5e3d3g5", "tanida"},
	{"f5f6e6d6c5e3d3g5d7", "aircraft"},
	{"f5f6e6d6c5e3d3g5d7", "feldborg"},
	{"f5f6e6d6c5e3d3g5e2b5", "sailboat"},
	{"f5f6e6d6c5e3e7", "cow bat"},
	{"f5f6e6d6c5e3e7", "bat"},
	{"f5f6e6d6c5e3e7", "cambridge"},
	{"f5f6e6d6c5e3e7c6d7f7c7f4g6e8d8c8g5", "bat (kling continuation)"},
	{"f5f6e6d6c5e3e7c7d7c6d3", "melnikov"},
 	{"f5f6e6d6c5e3e7c7d7c6d3", "bat (piau continuation 1)"},
	{"f5f6e6d6c5e3e7c7d7c6f7", "bat (piau continuation 2)"},
	{"f5f6e6d6c5e3e7f4f7", "bat (kling alternative)"},
	{"f5f6e6d6c5f4", "chimney"},
	{"f5f6e6d6c7c6d7", "hamilton"},
	{"f5f6e6d6c7f4", "lollipop"},
	{"f5f6e6d6d7", "raccoon dog"},
	{"f5f6e6d6e7", "heath"},
	{"f5f6e6d6e7", "tobidashi \"jumping out\""},
	{"f5f6e6d6e7f4", "heath-chimney"},
	{"f5f6e6d6e7f4", "\"mass-turning\""},
	{"f5f6e6d6e7g5c5", "heath-bat"},
	{"f5f6e6d6e7g5g4", "iwasaki variation"},
	{"f5f6e6d6e7g5g6e3c5c6d3c4b3", "mimura variation ii"},
	{"f5f6e6d6f7", "snake"},
	{"f5f6e6d6f7",  "peasant"},
	{"f5f6e6d6f7e3d7e7c6c5d3", "pyramid"},
	{"f5f6e6d6f7e3d7e7c6c5d3", "checkerboarding peasant"},
	{"f5f6e6d6g7", "x-square opening"},
	{NULL, NULL},
};

/** The list of french opening names */
static const PositionName NOM_POSITION[] = {
	{{0x0000000000001400, 0x0000001c1c1e0000}, "campagnarde cannibale"},
	{{0x0000000000080800, 0x0000001c1c140000}, "Heath chemine"},
	{{0x00000000000e0000, 0x0000001c1c100000}, "campagnarde colle"},
	{{0x00000000000e0000, 0x0000001c3c000000}, "campagnarde droite"},
	{{0x00000000001c0000, 0x00003e3838000000}, "cannibale p"},
	{{0x0000000000240000, 0x00001c1c3c000000}, "Ishii cannibale"},
	{{0x0000000000240000, 0x00001c3c7c000000}, "cannibale cannibalesque"},
	{{0x00000000002c1c20, 0x0000183c7c100000}, "dfense Tamenori finlandaise"},
	{{0x0000000002340800, 0x0000001c1c08100e}, "Heath chemine diagonale bton"},
	{{0x00000000040c0400, 0x0000101878000000}, "de Grey edmead"},
	{{0x0000000004181000, 0x0000003878200000}, "Heath Comp'oth groupe"},
	{{0x0000000004262020, 0x000008181a180808}, "Heath mirage"},
	{{0x00000000042e0800, 0x000008181a101000}, "Heath mystere"},
	{{0x0000000004340400, 0x0000081818081838}, "campagnarde clou"},
	{{0x000000000624343c, 0x00281e3c38180800}, "Rose bton"},
	{{0x0000000008080000, 0x0000001c30000000}, "italienne"},
	{{0x0000000008080000, 0x0000041830000000}, "inhumaine"},
	{{0x0000000008080000, 0x00001c1c14000000}, "chemine"},
	{{0x00000000080e0000, 0x0000001c14200000}, "campagnarde dcolle"},
	{{0x0000000008302000, 0x00000038f0001000}, "Heath au bord"},
	{{0x0000000008380000, 0x0000001c30000000}, "citadine"},
	{{0x0000000008380000, 0x0000041830000000}, "Ishii"},
	{{0x00000000083c0000, 0x0000383c30000000}, "Ishii Piau droite"},
	{{0x00000000083c0000, 0x00003c3830000000}, "Ishii Piau diagonale"},
	{{0x0000000012000000, 0x00003c1c0c0c0800}, "Heath Comp'oth cannibale"},
	{{0x00000000120c0400, 0x0000001c0c020800}, "Heath Marconi"},
	{{0x0000000014000000, 0x0000081808040000}, "Collay"},
	{{0x0000000014000000, 0x0000081808080000}, "perpendiculaire classique"},
	{{0x00000000140c0400, 0x0000001f0b000800}, "Heath diagonale au bord"},
	{{0x0000000014320000, 0x00000c1c28080000}, "Shaman diagonale Jensen"},
	{{0x00000000143c1000, 0x0000181e28000000}, "Rose plate"},
	{{0x00000000143c1000, 0x00001a1c28000000}, "Rose diagonale"},
	{{0x00000000162c343c, 0x00281e3c28100800}, "rose bton (interversion)"},
	{{0x0000000018040000, 0x0000087800000000}, "ganglion"},
	{{0x0000000018040000, 0x0000383820000000}, "Ishii colle"},
	{{0x0000000018040000, 0x0000387800000000}, "ishii droite"},
	{{0x0000000018080000, 0x0000001c00000000}, "perpendiculaire"},
	{{0x0000000018080000, 0x0000001c04040000}, "diagonale"},
	{{0x000000001a080000, 0x0000001c04060000}, "Heath d'vitement"},
	{{0x000000001a180808, 0x0000003804060000}, "Heath au bord extrieure"},
	{{0x000000001c000000, 0x0000001c00000000}, "parallle"},
	{{0x000000001c000000, 0x0000087800000000}, "ganglion"},
	{{0x000000001c1c0000, 0x0000283820000000}, "Brightwell extrieure"},
	{{0x000000001e0c0400, 0x0000003820100800}, "Heath Comp'oth Murakami"},
	{{0x000000001e1c1000, 0x0000003820000800}, "Heath Comp'oth gnrale"},
	{{0x00000000240c0400, 0x0000101818100000}, "de Grey classique"},
	{{0x00000000240c0400, 0x0000101818200000}, "de Grey de btonnage"},
	{{0x000000002c000000, 0x0000081810300000}, "tigre"},
	{{0x000000002c000000, 0x0000101810140000}, "Parker"},
	{{0x000000002c000000, 0x0000101810180000}, "chat"},
	{{0x000000002c000000, 0x0000103810080000}, "italienne"},
	{{0x000000002c0c0800, 0x0000083e10000000}, "avion"},
	{{0x000000002c3c0800, 0x0000187e10000000}, "Rose plate transverse"},
	{{0x000000002c3c0800, 0x0000583e10000000}, "Rose transverse"},
	{{0x000000002e100800, 0x0000003c100e0000}, "Heath chemine russe"},
	{{0x000000002e480000, 0x00083c3c10101000}, "Brightstein acclre"},
	{{0x000000002e7e0000, 0x00087cfc10001000}, "Brightstein Andersson au bord"},
	{{0x0000000032580000, 0x00083c3c0c040000}, "transverse compacte"},
	{{0x000000003a0e0800, 0x00001c3c04000400}, "Caspard"},
	{{0x000000003a240000, 0x00005c3c04020000}, "troka"},
	{{0x000000003a380000, 0x0000141c04040000}, "orque extrieure retarde"},
	{{0x000000003c1c0000, 0x0000383e00000000}, "compacte  plat"},
	{{0x000000003c240000, 0x00003a3c40800000}, "charriot au bord"},
	{{0x000000003c322000, 0x00007c7c40400000}, "Moorlin de bton"},
	{{0x000000003c3a0000, 0x0000247c00000000}, "charriot"},
	{{0x000000003d1f0500, 0x00003c7f02000000}, "Greenberg Polyakov Penloup"},
	{{0x000000003e200000, 0x00001c38001c0000}, "Brightwell anglaise Nicolet"},
	{{0x000000003e380000, 0x0000101e00040000}, "orque  plat"},
	{{0x000000003e380000, 0x00001c3c00040000}, "aile op'masse"},
	{{0x000000003e580000, 0x00083c3f00020000}, "transverse droite au bord"},
	{{0x000000005e180000, 0x00083c3c20400000}, "Rose Amadeus"},
	{{0x000000005e1c0000, 0x0008181820400000}, "Tanida diagonale"},
	{{0x000000006e080000, 0x00083c3c10101000}, "Rose Stenvenson"},
	{{0x000000006e1c0000, 0x0008181810200000}, "Tanida droite"},
	{{0x000000007e3c0000, 0x00181e3e00000000}, "plate tournante ferme (?)"},
	{{0x00000002031c1000, 0x00002c1c1c020000}, "Ishii nordique Vallund"},
	{{0x00000002041c0000, 0x0000201818200000}, "Ishii dcolle en coin"},
	{{0x00000002041c0000, 0x0000201838000000}, "Ishii lazard"},
	{{0x00000002041e1000, 0x00002c1c1a010000}, "Ishii bton"},
	{{0x0000000206161000, 0x00002c1c18080800}, "Ishii de sacrifice"},
	{{0x00000002063e0800, 0x00001c3c78000400}, "Caspard de 91"},
	{{0x0000000207141000, 0x00002c1c18080800}, "Ishii nordique Vallund du pion blanc"},
	{{0x000000020e580800, 0x00003c3870021000}, "chat Marconi"},
	{{0x00000002100c0400, 0x0000001c0f020800}, "Heath diagonale sur le prbord"},
	{{0x0000000210180000, 0x00080c3c0c040000}, "Heath Comp'oth sauvage"},
	{{0x0000000210180000, 0x00080c3c0e000000}, "Heath Comp'oth plate"},
	{{0x0000000210180000, 0x00082c1c0e000000}, "Heath Comp'oth prtendante"},
	{{0x000000021c180000, 0x00090f3d03010000}, "Heath Comp'oth plate, var. de bton"},
	{{0x00000002260e0800, 0x00001c3c18100400}, "Caspard retarde"},
	{{0x00000002646e0000, 0x00103e3c18101800}, "Brightstein Andersson centrale"},
	{{0x00000002741e0000, 0x00103e3c08200800}, "Brightstein cric diagonale"},
	{{0x000000027c3c0000, 0x00051f3d02000000}, "suite polyakov"},
	{{0x0000000400142000, 0x000008387c081000}, "Daunas Kyu Syu"},
	{{0x00000004001e0800, 0x000030383c000000}, "japonaise Suekuni"},
	{{0x0000000400281820, 0x000030383c140000}, "Popov murakami"},
	{{0x0000000400300000, 0x000030383c080000}, "Brightwell anglaise"},
	{{0x0000000400781000, 0x00000c7838000000}, "japonaise Juhem-Caspard"},
	{{0x0000000400781000, 0x00004c3838000000}, "japonaise corbeau"},
	{{0x0000000404103000, 0x00000038782c0800}, "Heath Comp'oth groupe moderne"},
	{{0x00000004041c0000, 0x0000303838000000}, "compacte principale"},
	{{0x0000000408100000, 0x0000083810080000}, "cheval"},
	{{0x0000000408281820, 0x0000343830140000}, "Popov sharalle"},
	{{0x000000040a1c0000, 0x00043c3a30000000}, "Brightwell d'attente transverse"},
	{{0x000000040c002000, 0x0000003870381810}, "Heath Comp'oth groupe, var. de bton"},
	{{0x000000040c043804, 0x0000183830780000}, "dfense Tamenori du double accs"},
	{{0x000000040c182400, 0x00003c7830200000}, "Rose Daunas tournante"},
	{{0x000000040c182800, 0x00003c7830200000}, "Murakami tournante"},
	{{0x000000040c1c0000, 0x00002c3830000000}, "crochet pseudo-rose"},
	{{0x000000040c1c0000, 0x0000303830200000}, "Shaman triangle"},
	{{0x000000040c1c0000, 0x0000383830200000}, "compacte d'opposition"},
	{{0x000000040c1c0000, 0x0000787830000000}, "Muller en coins dcale"},
	{{0x000000040c1c0000, 0x0010387830000000}, "Muller fuse"},
	{{0x000000040c242000, 0x0000181830181810}, "Greenberg classique (interversion)"},
	{{0x000000040c242804, 0x0000183830181010}, "rfutation Tamenori"},
	{{0x000000040c3c0800, 0x0000187830400000}, "Rose Murakami"},
	{{0x000000040c3c0800, 0x0000187870000000}, "rose plate tournante (rotating flat)"},
	{{0x000000040c3c0800, 0x0000583830400000}, "Rose Daunas"},
	{{0x000000040c581400, 0x00003c7830202000}, "tournante Tamenori"},
	{{0x000000040e1c2c7e, 0x00103c3830201000}, "Greenberg Polyakov Shastein"},
	{{0x0000000410300000, 0x000008182c080000}, "Inoue"},
	{{0x0000000414241000, 0x0000181828180800}, "Rose Greenberg"},
	{{0x0000000418100000, 0x0000283820080000}, "Parker groupe"},
	{{0x0000000418100000, 0x0000383804000000}, "Brightwell"},
	{{0x0000000418180000, 0x00000c7820000000}, "coup 8"},
	{{0x0000000418180000, 0x00002c3820000000}, "Rose"},
	{{0x00000004181c2000, 0x00000c7860a00000}, "scorpion fausse Piau"},
	{{0x00000004181c2000, 0x00000cf860200000}, "scorpion Jacpoth"},
	{{0x00000004181c2000, 0x00002c3860a00000}, "scorpion Piau de bton"},
	{{0x0000000418200000, 0x0000101804000000}, "inconnue"},
	{{0x000000041c000000, 0x00000e1800000000}, "campagnarde"},
	{{0x000000041c000000, 0x00020c1800000000}, "ouverture X"},
	{{0x000000041c043000, 0x0000181820780800}, "Greenberg Alard (interversion)"},
	{{0x000000041e100000, 0x0000383800040000}, "Brightwell retarde"},
	{{0x0000000424040000, 0x0000383818380000}, "Brightwell parisienne"},
	{{0x0000000428100800, 0x00001018160c0400}, "Heath Comp'oth orthodoxe"},
	{{0x0000000428202000, 0x0000003810181000}, "campagnarde Kitajima"},
	{{0x0000000428300000, 0x00000c7810000000}, "Shaman plate"},
	{{0x0000000428300000, 0x00004c3810000000}, "Shaman diagonale"},
	{{0x0000000428300000, 0x0000583810080000}, "romaine"},
	{{0x000000042c041000, 0x0000281810380800}, "Ishii elysenne"},
	{{0x000000042c141000, 0x0000283810080800}, "Ishii matignon"},
	{{0x000000042c2c0800, 0x0000583810101000}, "Rose Piau"},
	{{0x0000000430380000, 0x000010180c040000}, "Brightwell intrieure"},
	{{0x0000000430380000, 0x000030780c000000}, "crochet Tastet"},
	{{0x0000000436300000, 0x00001018080c0800}, "orque de Cambridge"},
	{{0x000000043a380000, 0x0000101804060000}, "orque diagonale"},
	{{0x000000043c0c0000, 0x0000303840100000}, "Tau don Quichotte"},
	{{0x00000004480c2000, 0x00000c7830301000}, "scorpion Wada"},
	{{0x00000004480c2000, 0x00002c3830301000}, "scorpion Piau centrale"},
	{{0x00000004580c0000, 0x00000c7820300000}, "nouveau scorpion"},
	{{0x00000004580c0000, 0x00002c3820300000}, "scorpion Piau-puget"},
	{{0x0000000458180000, 0x00083c3820400000}, "Rose Bruyninckx"},
	{{0x000000046c082800, 0x00003c7810301000}, "Murakami occidentale"},
	{{0x000000046c303804, 0x00003838104c0000}, "casse-tte"},
	{{0x000000047e3c0000, 0x00147a7900000000}, "tournante paradoxale"},
	{{0x00000004bcd88000, 0x00003cf840200000}, "Greenberg Polyakov de Barcelone"},
	{{0x00000006001c1000, 0x000024181e020000}, "Ishii Penloup"},
	{{0x000000060c0c2000, 0x0000201830301810}, "Ishii ancienne droite au bord"},
	{{0x000000060c143000, 0x0000201830280c00}, "Ishii ancienne de la mini-diagonale"},
	{{0x0000000636580000, 0x00083f3808060000}, "transverse trois croix de bton"},
	{{0x00000006640e0000, 0x00103c3818301800}, "Brightstein cric centrale"},
	{{0x0000000664620000, 0x00103c3818181800}, "Brightstein Andersson Morillon"},
	{{0x0000000674120000, 0x00103c3808280800}, "rose Brightstein"},
	{{0x0000000800281820, 0x0000323438140000}, "Popov le saout"},
	{{0x0000000800380000, 0x0000001038040000}, "raton laveur"},
	{{0x0000000800380000, 0x000000103c000000}, "diagonale classique"},
	{{0x0000000808240000, 0x0000383030100000}, "Ishii Piau"},
	{{0x0000000808280000, 0x0000001030100800}, "Peccerillo"},
	{{0x0000000808280000, 0x0000001030101000}, "Heath"},
	{{0x0000000808300000, 0x0000001030080400}, "coquelicot"},
	{{0x000000080c002000, 0x00000010101c1000}, "Sprague"},
	{{0x000000080c002000, 0x00000010103c0000}, "Hamilton"},
	{{0x000000080c2c0000, 0x0000383030101000}, "Shaman triangle transverse"},
	{{0x0000000810080000, 0x0000001008040000}, "diagonale"},
	{{0x0000000810320000, 0x00000c142c040000}, "Shaman diagonale Vallund"},
	{{0x0000000810701000, 0x0000207068000000}, "Heath d'vitement malgache"},
	{{0x0000000818080000, 0x0000107000100000}, "beta"},
	{{0x0000000818080000, 0x00003e1400000000}, "raton Shaman"},
	{{0x0000000818080000, 0x0010183400000000}, "Nicolet"},
	{{0x0000000818100000, 0x00040c3400000000}, "Bill - de Grey"},
	{{0x0000000818200000, 0x00001c1402000000}, "Ishii dcolle"},
	{{0x0000000818300000, 0x00000c1424000000}, "Shaman"},
	{{0x000000081c100000, 0x0010103420000000}, "Tanida inverse"},
	{{0x000000081c100000, 0x0010303420000000}, "Tanida"},
	{{0x000000081c101000, 0x00001c7420000000}, "Nicolet moderne"},
	{{0x000000081c1c0000, 0x0000383020202000}, "Shaman triangle mur"},
	{{0x0000000820000800, 0x000000101e1c1000}, "Heath Comp'oth centrale"},
	{{0x0000000820000800, 0x000000101e3c0000}, "Heath Comp'oth droite"},
	{{0x0000000820100800, 0x000000101e0c0400}, "Heath Comp'oth diagonale"},
	{{0x000000082c6e0000, 0x0000383010103c00}, "Moorlin du centre"},
	{{0x0000000830701000, 0x0000303048000000}, "Heath Comp'oth chinoise"},
	{{0x0000000834000000, 0x0000381008280000}, "Parker rectangulaire"},
	{{0x000000083c000000, 0x0030383400000000}, "campagnarde Kitajima"},
	{{0x000000083c001000, 0x00000010021f0c08}, "Heath variante 56"},
	{{0x000000083c060000, 0x0000001002181c18}, "Heath Kawazoe"},
	{{0x000000083c060000, 0x0000001002381c08}, "Heath Ka"},
	{{0x000000083c080800, 0x00003c1000040000}, "Kung plate"},
	{{0x000000083c200000, 0x0000103040100000}, "Cambridge"},
	{{0x000000083c380800, 0x00001c1000041000}, "tigre centrale d'Athnes"},
	{{0x0000000840181010, 0x000000103c600000}, "Heath au bord mdiane"},
	{{0x00000008483c2000, 0x0000003034401000}, "Heath Comp'oth Takizawa"},
	{{0x0000000858080000, 0x0000001020701000}, "Heath classique"},
	{{0x0000000858081010, 0x0000001020700800}, "Heath au bord diagonale"},
	{{0x0000000858280000, 0x0000005020101000}, "Heath Drumel"},
	{{0x000000085c100000, 0x0010303420200000}, "avion droit"},
	{{0x000000085c100000, 0x0010303420400000}, "avion diagonal"},
	{{0x000000085c1a0000, 0x00103c3420400000}, "transverse principale"},
	{{0x000000085c1c0000, 0x0000383020600000}, "aile de bton"},
	{{0x000000085e180000, 0x00103c3420400000}, "plate transverse principale"},
	{{0x000000086c0c0000, 0x0000383010301000}, "aile Feldborg"},
	{{0x0000000870380800, 0x000000100c041000}, "Heath Comp'oth Buridan"},
	{{0x0000000878180000, 0x0010707080200000}, "Heath Comp'oth Murakami au bord"},
	{{0x0000000878200000, 0x0000001004181000}, "Heath Comp'oth (bat)"},
	{{0x0000000c00300000, 0x00003030380c0000}, "tigre diagonale franaise"},
	{{0x0000000c00302000, 0x00003030380c1000}, "orque diagonale de lille"},
	{{0x0000000c04103000, 0x00001010782c0800}, "Heath Comp'oth plate, var. d'insertion"},
	{{0x0000000c04103000, 0x00002010782c0800}, "Heath Comp'oth prtendante, var. d'insertion"},
	{{0x0000000c04240000, 0x000030f078100000}, "tigre centrale grosse masse"},
	{{0x0000000c08081820, 0x0000303030342000}, "Popov murakami retarde"},
	{{0x0000000c08200000, 0x0000081034101000}, "Bhagat"},
	{{0x0000000c08200000, 0x00001c1010100000}, "Ishii transverse"},
	{{0x0000000c08200000, 0x0000303030181000}, "Brightwell orque"},
	{{0x0000000c08202000, 0x0000303030181800}, "orque diagonale Rossi"},
	{{0x0000000c08280800, 0x0000303030141000}, "Popov"},
	{{0x0000000c08280800, 0x0000303030141000}, "Popov (interversion)"},
	{{0x0000000c08300800, 0x00003030300c0400}, "Japon 93 (interversion)"},
	{{0x0000000c08300800, 0x00003030300c0400}, "tigre Japon 93"},
	{{0x0000000c0c002000, 0x0000201070381810}, "Heath Comp'oth prtendante, var. de bton"},
	{{0x0000000c0c082400, 0x0000387030301000}, "Rose Daunas Microb"},
	{{0x0000000c0c2c0000, 0x0000383030101000}, "compacte d'opposition franaise"},
	{{0x0000000c0e1c2c7e, 0x0014383030201000}, "Greenberg Polyakov grosse masse"},
	{{0x0000000c143e0000, 0x0000383028000c00}, "coccinelle moderne"},
	{{0x0000000c18300000, 0x0000383024000000}, "crochet en coin"},
	{{0x0000000c1c180800, 0x0000387020202020}, "Murakami Bhagat"},
	{{0x0000000c1c1c0000, 0x0000383020202000}, "compacte d'opposition Sakaguchi"},
	{{0x0000000c1e3c0000, 0x00281e3020000000}, "Rose tournante"},
	{{0x0000000c1f200000, 0x00001010000c1c00}, "Heath Bracchi"},
	{{0x0000000c20600000, 0x0000201058181010}, "Heath ancienne"},
	{{0x0000000c20600000, 0x000030305c100000}, "Penloup-Juhem croix du sud"},
	{{0x0000000c3c082800, 0x0000387040b01000}, "Murakami japonaise"},
	{{0x0000000c3e3c0000, 0x00181e3040800000}, "plate tournante ouverte (?)"},
	{{0x0000000c40001010, 0x000000103c7c0000}, "Heath Shaman"},
	{{0x0000000c40001818, 0x000000103c7c0000}, "Heath carr blanc"},
	{{0x0000000c40003010, 0x000000103c7c0800}, "Heath Shaman retarde"},
	{{0x0000000c48001818, 0x0000001034780400}, "Heath scorpion"},
	{{0x0000000c48c00000, 0x00007070b0300000}, "campagnarde milanaise"},
	{{0x0000000c58080000, 0x0000087020100000}, "cheval roques"},
	{{0x0000000c5c0c0000, 0x0000381020700000}, "Brightwell sablier"},
	{{0x0000000c5c2c0000, 0x0000385020100000}, "Brightwell d'attente"},
	{{0x0000000e0c2c0800, 0x0000181012100000}, "Heath Comp'oth Ralle"},
	{{0x0000000e58180000, 0x00003c3020600000}, "Kung bton"},
	{{0x0000000e6e240000, 0x0014783010181800}, "Brightstein clef anglaise"},
	{{0x0000001210100000, 0x00080c0c0c0c0800}, "Heath Comp'oth cano"},
	{{0x0000001214200000, 0x00101c0c0a0c0800}, "Panthon"},
	{{0x0000001217100000, 0x00000c0c080c0800}, "Heath chemine diagonale centrale"},
	{{0x00000012182c0000, 0x00101c0c06020000}, "Ishii ancienne"},
	{{0x0000001400102000, 0x000010287c0c0000}, "chat en damier"},
	{{0x0000001414100000, 0x00000808080e0000}, "campagnarde transverse"},
	{{0x0000001416300000, 0x00081e0828080000}, "japonaise Juhem-Caspard"},
	{{0x0000001418100000, 0x0000382800080000}, "cheval crochet"},
	{{0x00000014181c2000, 0x00003ce860200000}, "scorpion Jacpoth retard"},
	{{0x0000001420100800, 0x000818081e0c0400}, "Heath Comp'oth damier"},
	{{0x0000001420600000, 0x000018285c100000}, "Penloup-Juhem Kaneda"},
	{{0x0000001420701000, 0x0000182858080000}, "Heath Comp'oth Brightwell"},
	{{0x0000001436000000, 0x0000080a080e0000}, "campagnarde berner"},
	{{0x0000001436010000, 0x0000080b080e0000}, "campagnarde danoise"},
	{{0x00000014580c0000, 0x00003c6820300000}, "scorpion group"},
	{{0x0000001470300800, 0x000008280c0c1400}, "campagnarde Ralle"},
	{{0x0000001474540000, 0x0000242808280800}, "zbre"},
	{{0x000000160c5c0000, 0x00003c2832200000}, "Murakami doyo-ka"},
	{{0x0000001610100000, 0x000c1c2808080000}, "in my country tire-bouchon"},
	{{0x0000001610240000, 0x00101c080c080000}, "Ishii des danseurs"},
	{{0x0000001630300000, 0x00080c080c0c0800}, "Shaman plate ancienne"},
	{{0x00000016385c0000, 0x00003c2806020000}, "Murakami Tashii"},
	{{0x00000018180c0000, 0x0000382024000000}, "compacte en coin"},
	{{0x000000181a100000, 0x00040c06040e0000}, "campagnarde Shibata"},
	{{0x000000181c1c0000, 0x0000382020200000}, "Brightwell aile"},
	{{0x000000181c1e0800, 0x0000382020202400}, "Moorlin"},
	{{0x000000181c1e0800, 0x0000382020202400}, "moorlin retarde"},
	{{0x000000181c2c0000, 0x00101c0702000000}, "Ishii nordique"},
	{{0x000000181c2c0000, 0x00101e0602000000}, "Ishii dcolle classique"},
	{{0x000000183a010000, 0x00000407040e0000}, "campagnarde ???"},
	{{0x0000001a78180000, 0x00003c2402200000}, "Kung du pion isol"},
	{{0x0000001c00103000, 0x000010207c680000}, "triangle Nicolet centrale"},
	{{0x0000001c00200000, 0x000010207c100000}, "grande tigre centrale"},
	{{0x0000001c00302000, 0x000010207c400000}, "triangle centrale"},
	{{0x0000001c00400000, 0x000010207c302000}, "Penloup-Juhem, dfense Tastet"},
	{{0x0000001c00600000, 0x000010607c100000}, "Penloup-Juhem requiem"},
	{{0x0000001c04200000, 0x00000000380e0000}, "campagnarde nordique"},
	{{0x0000001c08501000, 0x0000102070282000}, "Heath Comp'oth informatique"},
	{{0x0000001c10200000, 0x000000000e0e0000}, "campagnarde traineau"},
	{{0x0000001c143e0000, 0x0008182028000c00}, "coccinelle classique"},
	{{0x0000001c18100800, 0x0000302024600000}, "dcale  plat"},
	{{0x0000001c18101000, 0x0000302024600000}, "in my country a plat"},
	{{0x0000001c18103000, 0x0000302024680000}, "triangle Nicolet  plat"},
	{{0x0000001c18180000, 0x0000000006040400}, "Heath chemine droite"},
	{{0x0000001c18180000, 0x0000000006040800}, "Heath chemine diagonale"},
	{{0x0000001c18300000, 0x00081e0024000000}, "groupe japonaise"},
	{{0x0000001c38100000, 0x00000e2000080000}, "compacte extrieure"},
	{{0x0000001c38100000, 0x000070e040200000}, "campagnarde classique ou 11 au bord"},
	{{0x0000001c3a100000, 0x00041c2000080000}, "variante du triplet"},
	{{0x0000001c3c000000, 0x00000003020e0000}, "campagnarde 9 au bord"},
	{{0x0000001c3c000000, 0x00000e2000080000}, "petite tigre centrale"},
	{{0x0000001c3c2c0000, 0x0000386040900000}, "Brightwell d'attente au bord"},
	{{0x0000001c3e580000, 0x00083c2201020000}, "transverse go"},
	{{0x0000001c58080000, 0x0000386020100000}, "roques Caspard"},
	{{0x0000001e00340000, 0x00080c201c080800}, "ishii T blanc"},
	{{0x0000001e38100000, 0x00041c2000080000}, "variante Aldaron"},
	{{0x0000002220240000, 0x00001c1c1e080000}, "tigre centrale Murakami"},
	{{0x0000002220240000, 0x00002c1c1e080000}, "tigre centrale Plowman"},
	{{0x0000002220300000, 0x00001c1c1e0c0800}, "tigre centrale japonaise"},
	{{0x0000002400080000, 0x00000c1838100000}, "tigre compacte"},
	{{0x0000002400080000, 0x000010181c100000}, "Mimura"},
	{{0x0000002400080000, 0x00001c1818100000}, "tigre centrale"},
	{{0x00000024000c0400, 0x00001c1818300800}, "tigre diagonale Ralle"},
	{{0x0000002400120800, 0x000030181c0c0400}, "japonaise de la roussette"},
	{{0x0000002400160000, 0x000030181c080808}, "japonaise douteuse"},
	{{0x0000002400180800, 0x000030181c040000}, "tigre diagonale groupe"},
	{{0x00000024001c0000, 0x000030181e000000}, "compacte de Grey"},
	{{0x00000024040c0000, 0x0000301818100000}, "Shaman tau"},
	{{0x00000024080c0000, 0x00000c1830300000}, "tigre scorpion"},
	{{0x0000002408180000, 0x00000c1830200000}, "Ralle"},
	{{0x0000002408180000, 0x0000181834000000}, "Muller en coins"},
	{{0x0000002408180800, 0x0000341810040000}, "tigre diagonale Holgersson"},
	{{0x000000240c080400, 0x00000c1830300000}, "romaine connexe"},
	{{0x0000002414040000, 0x0000181808180800}, "chat amricaine"},
	{{0x0000002414081000, 0x0000181868302000}, "Heath Comp'oth Murakami au centre"},
	{{0x0000002418080000, 0x0000381804100000}, "Maruoka"},
	{{0x000000241a300000, 0x00003018040e0000}, "scorpion classique"},
	{{0x000000242c200000, 0x00000c1810181000}, "Tau cerf"},
	{{0x0000002600040000, 0x000018183e080000}, "tigre centrale d'interversion"},
	{{0x0000002630180000, 0x00003c180e060000}, "Greenberg classique"},
	{{0x0000002a00080000, 0x00000e143e040000}, "campagnarde damier"},
	{{0x0000002a30380000, 0x000024140c040000}, "cannibale centrale"},
	{{0x0000002c00080000, 0x000008103c000000}, "cheval centrale"},
	{{0x0000002c00100000, 0x000008103c000000}, "ball"},
	{{0x0000002c00100000, 0x00003010180c0000}, "tigre diagonale"},
	{{0x0000002c00180000, 0x000008103c040000}, "Muller d't symtrique"},
	{{0x0000002c00200000, 0x00003010181c1000}, "tigre diagonale damier"},
	{{0x0000002c04040000, 0x0000301018080000}, "Ishii tigre"},
	{{0x0000002c08000800, 0x00003010103c0000}, "tigre diagonale Hashimoto"},
	{{0x0000002c08080000, 0x0000081030100000}, "cheval transversale"},
	{{0x0000002c08100000, 0x0000181034000000}, "cheval Muller"},
	{{0x0000002c08100800, 0x00003010100c0400}, "tigre diagonale Tom-Pouce"},
	{{0x0000002c18083020, 0x0000f85064700000}, "Tamenori franaise penloup"},
	{{0x0000002c18083020, 0x0010b85064700000}, "Tamenori franaise shaman"},
	{{0x0000002c18302000, 0x0000181024400000}, "Tamenori"},
	{{0x0000002c1a300000, 0x00043812040c0000}, "scorpion excentr"},
	{{0x0000002c1c180000, 0x0000b85060600000}, "dfense Tamenori"},
	{{0x0000002c38080000, 0x04083c1000100000}, "Andersson, var. Cali"},
	{{0x0000002c58080000, 0x0000081020500000}, "cheval Daunas"},
	{{0x0000002c70080000, 0x001038500c000000}, "roques argentine"},
	{{0x0000002e04080000, 0x00001c1038000000}, "tigre centrale baltique"},
	{{0x0000002e08080000, 0x00040c1030200000}, "Kung diagonale"},
	{{0x0000002e08200000, 0x00001c1016100000}, "Ishii T noir"},
	{{0x0000002e34080000, 0x00003c100a040000}, "Greenberg classique (interversion)"},
	{{0x0000002e64000400, 0x00101e1018380800}, "chat Andriani"},
	{{0x0000002e6c2c0000, 0x0018781012111000}, "plate tournante de Tchliabinsk"},
	{{0x0000002f0b200000, 0x00001010141c1c00}, "Heath japonaise"},
	{{0x0000002f13200000, 0x000030100c0c1c00}, "Heath en coin"},
	{{0x0000003200040000, 0x00000c0c3e080000}, "tigre centrale de Paris"},
	{{0x000000323d080000, 0x00001c0c02070000}, "Greenberg semi-originelle"},
	{{0x0000003713200000, 0x000018080c0c1c00}, "Heath standard"},
	{{0x0000003a1c0c0000, 0x0000760400100000}, "bicorne"},
	{{0x0000003c00040000, 0x000020001e080000}, "Ishii Heath"},
	{{0x0000003c0c040000, 0x0000000030382c00}, "campagnarde Taniguchi"},
	{{0x0000003c0c040000, 0x0000000030382c00}, "campagnarde Tanigushi"},
	{{0x0000003c1c000400, 0x00000800201e0000}, "triangle Nicolet"},
	{{0x0000003c1c000800, 0x00000800201e0000}, "geisha"},
	{{0x0000003c1c001000, 0x00000800201e0000}, "geisha dcale"},
	{{0x0000003c1c080800, 0x0000100060343008}, "Heath Comp'oth Murakami au bord"},
	{{0x0000003c1e180000, 0x00103c0201020000}, "Bruyninckx d'encerclement"},
	{{0x0000003c3c1c0000, 0x0000244080000000}, "cannibale 11 au bord"},
	{{0x0000003c3e3c0000, 0x00185e4040800000}, "plate tournante japonaise"},
	{{0x0000005e2e480000, 0x00087c2010141000}, "Brightstein Tastet"},
	{{0x0000006e26480000, 0x00083c1018161000}, "Brightstein retarde (interversion)"},
	{{0x0000006e2e480000, 0x00083c1010161000}, "Brightstein retarde"},
	{{0x0000007e3c1a0000, 0x00307c80c0e00000}, "Rose fautive"},
	{{0x00000102007a1800, 0x00003c7c3e010000}, "plate tournante plbenne"},
	{{0x00000102024a1800, 0x00003c7c3c302000}, "plate tournante kaneda"},
	{{0x00000102026a1000, 0x00003c7c3c100804}, "plate tournante, rfutation tamenori"},
	{{0x0000010256681000, 0x00003c7c28160804}, "semi-Tamenori"},
	{{0x0000010266701800, 0x00003c7c180e0400}, "cygne"},
	{{0x0000010274781800, 0x00003c7c0a070000}, "plate tournante ferme, var. ouverte"},
	{{0x000001028e4e2800, 0x00003c7c30301000}, "paradoxale russie"},
	{{0x000001028e5e0800, 0x00003c7c30202020}, "paradoxale de sacrifice"},
	{{0x0000010a260e0800, 0x00003c3418100400}, "Tamenori de Londres"},
	{{0x0000010a3e3e0000, 0x0000247401010000}, "cannibale Tezuka"},
	{{0x0000011830020000, 0x00080c060f1c0000}, "Nicolet historique"},
	{{0x0000013315030000, 0x00000e0c0a0c0000}, "campagnarde ???"},
	{{0x0000020010380400, 0x00043c1e2c000000}, "scorpion du pion isol"},
	{{0x00000200203c0800, 0x0000583e1e000000}, "transverse du pion isol"},
	{{0x0000020028000800, 0x00040c3e14080400}, "equerre damier"},
	{{0x0000020030080800, 0x00040c3e0c040000}, "equerre centrale"},
	{{0x0000020038380000, 0x0004141e04040000}, "Tau"},
	{{0x000002003c3c1000, 0x0000187e02000000}, "Amadeus du hbleur"},
	{{0x000002007e3c0000, 0x0014787e00000000}, "tournante mange"},
	{{0x00000200fe3c0000, 0x00187c7e00c00000}, "johnson"},
	{{0x00000202183c0000, 0x00041c3c26000000}, "Greenberg alard (interversion)"},
	{{0x000002040c3c0800, 0x0000583a70000000}, "transverse pineuse"},
	{{0x0000020410380400, 0x00083c1a2c000000}, "scorpion du Luxembourg"},
	{{0x0000020414241000, 0x0000181a28180800}, "Greenberg retarde"},
	{{0x00000204283c0800, 0x0000583a14020000}, "transverse trois croix"},
	{{0x00000204383c1000, 0x0000187a04020000}, "Amadeus Mercedes"},
	{{0x000002043e3c0000, 0x0014787840800000}, "tournante tourniquet"},
	{{0x0000020472180000, 0x00083c380c060000}, "plate transverse classique"},
	{{0x00000206003c1000, 0x000018183f020000}, "Bruyninckx Mercedes koala"},
	{{0x0000020616241000, 0x00001818281a0800}, "Bruyninckx Mercedes nordique"},
	{{0x0000020a183c0000, 0x00041c3426000000}, "Greenberg Alard"},
	{{0x0000021038580000, 0x00083c2c06030000}, "transverse trois croix de l'arche"},
	{{0x0000021038580000, 0x00083c2e06020000}, "transverse trois croix mercedes"},
	{{0x0000021078180000, 0x00083c2c06030000}, "plate transverse de l'arche"},
	{{0x0000021078180000, 0x00083c2e06020000}, "plate transverse mercedes"},
	{{0x0000021630100000, 0x000c0c280c080000}, "compacte triangle qvist"},
	{{0x0000021638100000, 0x00041c2804080000}, "compacte triangle de Varsovie"},
	{{0x0000022000080000, 0x00040c1c38100000}, "compacte triangle"},
	{{0x0000022000080000, 0x00040c1c3c000000}, "triangle"},
	{{0x00000220040c0000, 0x00080c1e38000000}, "in my country de Lyon"},
	{{0x0000022008180000, 0x00040c1e34000000}, "variante de Stockholm"},
	{{0x0000022018180000, 0x00103c1c06030000}, "Bruyninckx arche"},
	{{0x0000022018180000, 0x00103c1e06020000}, "Bruyninckx Mercedes"},
	{{0x0000022400180000, 0x00040c183c040000}, "Tamenori classique"},
	{{0x0000022412180000, 0x00103c180c060000}, "Bruyninckx classique"},
	{{0x0000022635080000, 0x00003c180a070000}, "Greenberg originelle (interversion)"},
	{{0x0000022664020000, 0x00183c58183c0000}, "doyo-ka centrale"},
	{{0x0000022a00180000, 0x00041c143c040000}, "coccinelle thor"},
	{{0x0000022c00100000, 0x000418123c080000}, "aberration optique diagonale"},
	{{0x0000022e04080000, 0x00041c1038000000}, "coccinelle"},
	{{0x0000023400100000, 0x0008180a3c080000}, "aberration optique droite"},
	{{0x0000023618180000, 0x00041c0824040000}, "Tamenori retarde Di Meglio"},
	{{0x0000023a00080000, 0x00040c043c040000}, "triangle centrale classique"},
	{{0x0000023a18180000, 0x00040c0426040000}, "tamenori retarde Penloup-Caspard"},
	{{0x000002446e302000, 0x00187838100c1000}, "Brightstein Lazard"},
	{{0x000002646a302000, 0x00183818140e1000}, "Brightstein Piau (interversion)"},
	{{0x0000026c0e0a0000, 0x00003c1070741000}, "doyo-ka polonaise"},
	{{0x000003013f2e1800, 0x00003c7e40d02000}, "feinder"},
	{{0x000003017f2e0800, 0x00003c7e00503010}, "jotel"},
	{{0x00000302003c1000, 0x0000181c3e020000}, "Bruyninckx arche tortue"},
	{{0x00000303013f0800, 0x00045c3c7e000000}, "Rose brisson"},
	{{0x0000030614241000, 0x00001818281a0800}, "Bruyninckx arche Ralle"},
	{{0x0000032638380000, 0x0000341805040000}, "cannibale diagonale Maruoka"},
	{{0x0000032e00380000, 0x000024107d040000}, "cannibale diagonale transverse"},
	{{0x00000400002c1030, 0x0000383c38100804}, "cannibale Murakami"},
	{{0x0000040010300000, 0x0000383e0c080000}, "Lepidopterix niger"},
	{{0x0000040010300000, 0x0008083c08080000}, "Kung"},
	{{0x000004001c3c0800, 0x0000583e60000000}, "Rose transverse retarde"},
	{{0x0000040020322000, 0x0000283e1c081000}, "scorpion des 2 pions isols"},
	{{0x0000040036300000, 0x0000101e080c0800}, "Popov brisson"},
	{{0x0000040038200000, 0x0000101e04100000}, "tigre diagonale classique"},
	{{0x0000040038300000, 0x0000583e00080000}, "romaine Handel"},
	{{0x000004003c380000, 0x0000101e02050000}, "Popov au bord"},
	{{0x0000040114240000, 0x0000383e2a010000}, "cannibale classique"},
	{{0x000004021c3c0000, 0x00011b3d22000000}, "Greenberg polyakov"},
	{{0x000004040024343c, 0x002818383f180800}, "Rose tardy"},
	{{0x0000040410300000, 0x000038382c080000}, "Lepidopterix albus"},
	{{0x0000040414300000, 0x00080838080c0000}, "suite franaise"},
	{{0x0000040510240000, 0x000038382e030000}, "cannibale diagonale"},
	{{0x00000408103c0000, 0x000008372e000000}, "Greenberg wahlberg"},
	{{0x000004081e3c0000, 0x0030183620000000}, "Rose Piau classique"},
	{{0x0000040820322000, 0x000028341e081000}, "scorpion de Varsovie"},
	{{0x0000040c00302000, 0x000010107c040000}, "Kung diagonale, variante 33"},
	{{0x0000040c003c1020, 0x000028303c400000}, "scorpion Piau-Puget, var. de bton"},
	{{0x0000040c08202000, 0x0000101074100800}, "Kung diagonale, variante 42"},
	{{0x0000040c0c243020, 0x0000183030180c10}, "Greenberg originelle"},
	{{0x0000040c1c3c3820, 0x043c387060000000}, "plate tournante ferme, var. ferme"},
	{{0x0000040c30200000, 0x000010100c140000}, "tigre diagonale moderne"},
	{{0x0000040c3c0c0800, 0x0000583040301000}, "Rose Piau-Marconi"},
	{{0x0000040c5e2c0000, 0x0028187020101000}, "rose Collay"},
	{{0x0000041064740000, 0x0000086818083c00}, "Ishii, var. d'vitement acclre"},
	{{0x0000041410300000, 0x000818082c080000}, "groupe anglaise "},
	{{0x0000041434100000, 0x00080828080c0000}, "Brightwell Leader"},
	{{0x0000041818300000, 0x0008180426000000}, "tigre diagonale milanaise"},
	{{0x000004182c7c0000, 0x0000006050803c00}, "Ishii, var. d'vitement"},
	{{0x00000420000e0000, 0x0000181e3e102000}, "Penloup-Juhem Ohyanagi"},
	{{0x0000042000120000, 0x0000181e3e0c0400}, "Penloup-Juhem de 92"},
	{{0x00000424000c0400, 0x000018183e020000}, "Tamenori retarde"},
	{{0x0000043c04000400, 0x000008003a0e0800}, "triangle grosse masse"},
	{{0x000004402e302000, 0x0018783c10081400}, "Brightstein Tastet 2 contre 1"},
	{{0x0000044c00100000, 0x000038303c080000}, "Brightwell anglaise Robin"},
	{{0x0000044c06242000, 0x0028383038181800}, "dfense danoise"},
	{{0x0000045c06042000, 0x0028182078381800}, "dfense Bhagat"},
	{{0x0000045c0e0c0000, 0x0028182070301000}, "Brightstein Bhagat"},
	{{0x0000045c0e2c0000, 0x0028382030101000}, "Brightstein cric"},
	{{0x000004645e0c0400, 0x0028181820721800}, "Brightstein Torben"},
	{{0x0000047406240000, 0x0028180838181800}, "Brightstein classique"},
	{{0x000004747c280400, 0x0028180902161800}, "Brightstein Piau"},
	{{0x0000047c0e0c0000, 0x0028180030303000}, "Brightstein Andersson"},
	{{0x00000520040e0000, 0x0000181f3a102000}, "Penloup-Juhem classique"},
	{{0x0000060430300000, 0x0000101a0c0c0800}, "suite de bton"},
	{{0x00000608383c0800, 0x0000583606020000}, "transverse trois croix classique"},
	{{0x0000062000080000, 0x0008081c38100000}, "compacte extrieure droite"},
	{{0x0000062000080000, 0x0008081c3c000000}, "in my country"},
	{{0x0000062000080000, 0x0010081c38100000}, "compacte extrieure diagonale"},
	{{0x0000062000080000, 0x0010081c3c000000}, "tigre centrale dcale"},
	{{0x00000620000c0000, 0x0008081c3e000000}, "in my country alien"},
	{{0x00000624000c0000, 0x000808183c020000}, "in my country Jensen"},
	{{0x0000070a3c3c0000, 0x0018787442810000}, "plate tournante ferme, var. ouverte"},
	{{0x0000080020120000, 0x0000003c1c0c0c00}, "geisha pseudo-Nicolet"},
	{{0x00000800201a0000, 0x0000003c1e040800}, "geisha Ghirardato"},
	{{0x00000800220e0800, 0x0000043c1c100400}, "triangle de Varsovie"},
	{{0x000008002a0e0800, 0x0020143e14000400}, "Tamenori anglaise"},
	{{0x00000800380e0800, 0x0000043c06010400}, "triangle grosse masse"},
	{{0x000008003a080000, 0x0000043c04040000}, "tigre centrale Andersson"},
	{{0x000008003e080000, 0x0000003e00040000}, "tigre centrale classique"},
	{{0x0000080420300000, 0x000010181c0c0800}, "crochet intrieur"},
	{{0x000008043a080000, 0x0000003804060000}, "Penloup-Juhem"},
	{{0x0000080c30380000, 0x000030100c040000}, "Brightwell papillon"},
	{{0x0000081034001000, 0x0000000c08380000}, "Bhagat, var.principale"},
	{{0x000008163a102000, 0x00201428040e0000}, "Penloup-Juhem ancienne"},
	{{0x000008183a100000, 0x0004042404080000}, "variante du quadruplet"},
	{{0x000008183c1e2800, 0x3c3c342640000000}, "Rose bton colle"},
	{{0x0000081a38100000, 0x0004042404080000}, "variante des deux diagonales"},
	{{0x0000081c3e1c0c00, 0x000070e040a03020}, "chat Penloup"},
	{{0x0000081c7e0c0400, 0x0000706080303020}, "chat glisse au bord"},
	{{0x0000081e28300000, 0x00000420140c0800}, "Shaman plate moderne"},
	{{0x00000820160c1c20, 0x0000101c28720000}, "Tamenori de sophia"},
	{{0x00000820340c1c20, 0x0000101c0a330000}, "Tamenori franaise"},
	{{0x000008203e040400, 0x00001018003a1804}, "Tamenori japonaise"},
	{{0x000008280c081000, 0x0000101430100800}, "roques Penloup antisymtrique"},
	{{0x000008280c081000, 0x0000101430102000}, "Daunas aberration optique"},
	{{0x000008382c000800, 0x00001000103c0400}, "Penloup-Juhem Andriani"},
	{{0x000008540a003010, 0x00007028343e0000}, "chat Svirskiy"},
	{{0x00000a1f08580800, 0x0004142074202400}, "romaine des minis diagos"},
	{{0x00000a3c38100800, 0x00001000042c1420}, "Penloup-Juhem no-Tastet"},
	{{0x00000c6038000400, 0x0000101c04781000}, "japonaise Suekuni"},
	{{0x0000122c5e0c0400, 0x0004e8d0a0f01000}, "Rose bton d'extraction"},
	{{0x00002406540c0400, 0x0018183828502800}, "Brightstein Tastet ko"},
	{{0x0000560014081000, 0x080c087c28302000}, "chat russe"},
	{{0x000404740a041020, 0x0010390a343a0800}, "Tchliabinsk diagonale au bord"},
	{{0x0004047600041020, 0x001038083f3a0800}, "Tchliabinsk droite au bord"},
	{{0, 0}, NULL}
};

/** The list of french opening names */
static const PositionName POSITION_NAME[] = {
	{{0x0000000000080800, 0x0000001c1c140000}, "heath-chimney"},
	{{0x0000000000080800, 0x0000001c1c140000}, "\"mass-turning\""},
	{{0x0000000000240000, 0x00001c3c7c000000}, "Tanida buffalo"},
	{{0x0000000008080000, 0x00001c1c14000000}, "chimney"},
	{{0x0000000008102000, 0x0000001c140c0000}, "lollipop"},
	{{0x0000000008380000, 0x0000041830000000}, "buffalo, Kenichi variation"},
	{{0x000000000e100000, 0x0000003810200000}, "swallow"},
	{{0x00000000143c1000, 0x0000181e28000000}, "flat"},
	{{0x0000000018040000, 0x0000087800000000}, "ganglion"},
	{{0x0000000018080000, 0x0000001c00000000}, "perpendicular opening"},
	{{0x0000000018180800, 0x00001c7e00000000}, "sailboat"},
	{{0x000000001c000000, 0x0000001c00000000}, "parallel opening"},
	{{0x000000001c000000, 0x0000087800000000}, "no-cat"},
	{{0x000000001e1c1000, 0x0008283820200000}, "no-cat (continuation)"},
	{{0x000000001e1e1800, 0x00003c7e20400000}, "rotating flat (Kling continuation)"},
	{{0x000000002c000000, 0x0000081810300000}, "tiger"},
	{{0x000000002c000000, 0x0000101810180000}, "cat"},
	{{0x000000002c000000, 0x0000103810080000}, "italian"},
	{{0x000000002c0c0800, 0x0000083e10000000}, "aircraft, Feldborg"},
	{{0x000000002c3c0800, 0x0000583e10000000}, "Rose-Birdie, Rose-Tamenori"},
	{{0x0000000032580000, 0x00083c3c0c040000}, "Rose-Tamenori-Kling"},
	{{0x00000002041c0000, 0x0000201818200000}, "Maruoka buffalo"},
	{{0x0000000408100000, 0x0000083810080000}, "horse"},
	{{0x000000040c3c0800, 0x0000187830400000}, "Murakami variation"},
	{{0x0000000410300000, 0x000008182c080000}, "Inoue"},
	{{0x0000000414241000, 0x0000181828180800}, "Dawg"},
	{{0x0000000414241000, 0x0000181828180800}, "Greenberg"},
	{{0x0000000418100000, 0x0000383804000000}, "Brightwell"},
	{{0x000000041c000000, 0x00000e1800000000}, "peasant, snake"},
	{{0x000000041c000000, 0x00020c1800000000}, "x-square opening"},
	{{0x000000042c040000, 0x0000083810080000}, "Rose-v-Toth"},
	{{0x0000000674120000, 0x00103c3808280800}, "Brightstein"},
	{{0x0000000800380000, 0x000000103c000000}, "cow"},
	{{0x0000000808280000, 0x0000001030100800}, "raccoon dog"},
	{{0x0000000808280000, 0x0000001030101000}, "Heath, Tobidashi \"jumping out\""},
	{{0x00000008082c0000, 0x0000001030100c00}, "Hamilton"},
	{{0x0000000810080000, 0x0000001008040000}, "diagonal opening"},
	{{0x0000000818080000, 0x0000203020000000}, "wing variation"},
	{{0x0000000818080000, 0x0000381000000000}, "semi-wing variation"},
	{{0x0000000818100000, 0x00040c3400000000}, "Aubrey, Tanaka"},
	{{0x0000000818300000, 0x00000c1424000000}, "danish, Shaman"},
	{{0x000000081c100000, 0x0010303420000000}, "Tanida, Rose-Birth"},
	{{0x000000081c380000, 0x0000103060400000}, "bat (Kling alternative)"},
	{{0x000000081e180000, 0x00103c3420000000}, "rotating flat"},
	{{0x000000083c200000, 0x0000103040100000}, "bat, cow bat, Cambridge"},
	{{0x0000000858280000, 0x0000005020101000}, "Iwasaki variation"},
	{{0x0000000878200000, 0x0000001004181000}, "Heath-bat"},
	{{0x0000000c08200000, 0x0000081034101000}, "Bhagat"},
	{{0x0000000c08200000, 0x00001c1010100000}, "Hokuriku buffalo"},
	{{0x0000000c2c040000, 0x0000383010080000}, "Landau"},
	{{0x0000001418100000, 0x00003c2820000000}, "Iago, Rose"},
	{{0x0000001c1c1c1000, 0x0000382020200800}, "Ishii"},
	{{0x0000002400080000, 0x00000c1838100000}, "Leader's tiger"},
	{{0x0000002400080000, 0x000010181c100000}, "Mimura"},
	{{0x0000002408180000, 0x00000c1830200000}, "Ralle"},
	{{0x000000240c080400, 0x00000c1830300000}, "Sakaguchi"},
	{{0x0000002a00080000, 0x00000e143e040000}, "checkerboarding peasant, pyramid"},
	{{0x0000002c00100000, 0x00003010180c0000}, "Stephenson"},
	{{0x0000002c18302000, 0x0000181024400000}, "Tamenori"},
	{{0x0000003c1c060000, 0x00000e0202080000}, "bat (Piau continuation 2)"},
	{{0x0000011335120000, 0x00000e0c0a0c0c00}, "bat (Kling continuation)"},
	{{0x0000022000080000, 0x00040c1c3c000000}, "Rose-Bill"},
	{{0x0000040010300000, 0x0008083c08080000}, "Kung"},
	{{0x0000040038200000, 0x0000101e04100000}, "no-Kung"},
	{{0x00000400383a0000, 0x0000101e06050400}, "no-Kung (continuation)"},
	{{0x0000040c30200000, 0x000010100c140000}, "Comp'oth"},
	{{0x0000040c38381820, 0x0000283040c02000}, "F.A.T. draw"},
	{{0x0000042018080000, 0x0000381e04100000}, "Maruoka"},
	{{0x0000042c00060000, 0x000008123e080000}, "bat (Piau continuation 1), Melnikov"},
	{{0x0000044400100000, 0x001078381c0a0000}, "Mimura variation ii"},
	{{0x000008143a300000, 0x00000028040a0400}, "Berner"},
	{{0x000024047c241000, 0x0034183800180800}, "mainline tiger"},
	{{0,0}, NULL},
};

/**
 * Compare two string, ignoring accents, font case, etc.
 *
 * @param s1 first string.
 * @param s2 second string.
 */
static bool match(const char *s1, const char *s2)
{
	int c1, c2;

	for (;;) {
		while (isspace(*s1) || ispunct(*s1)) ++s1;
		while (isspace(*s2) || ispunct(*s2)) ++s2;
	
		if (*s1 == '\0' || *s2 == '\0') return (*s2 == '\0');

		c1 = tolower(*s1);
		c2 = tolower(*s2);
		if (c1 == '' || c1 == '' || c1 == '') c1 = 'e';
		if (c1 == '') c1 = 'a';
		if (c1 == '') c1 = 'i';
		if (c2 == '' || c2 == '' || c2 == '') c2 = 'e';
		if (c2 == '') c2 = 'a';
		if (c2 == '') c2 = 'i';

		if (c1 != c2) return false;

		++s1; ++s2;
	}
}

/**
 * @brief Translate an opening name into its move sequence.
 *
 * @param opening_name An opening name (in lowercase).
 * @return A move sequence.
 */
const char *opening_get_line(const char *opening_name)
{
	const OpeningName *o;

	for (o = OPENING_NAME; o->name != NULL; ++o) {
		if (match(opening_name, o->name)) break;
	}
	return o->line;
}

/**
 * @brief Check if a board has got an engilsh opening name.
 *
 * @param board Board.
 * return An opening name, or NULL if none has been found.
 */
const char *opening_get_english_name(const Board *board)
{
	const PositionName *p;
	Board unique[1];

	board_unique(board, unique);
	for (p = POSITION_NAME; p->name != NULL; ++p) {
		if (board_equal(unique, &p->board)) break;
	}

	return p->name;
}

/**
 * @brief Check if a board has got a french opening name.
 *
 * @param board Board.
 * return An opening name, or NULL if none has been found.
 */
const char *opening_get_french_name(const Board *board)
{
	const PositionName *p;
	Board unique[1];

	board_unique(board, unique);
	for (p = NOM_POSITION; p->name != NULL; ++p) {
		if (board_equal(unique, &p->board)) break;
	}

	return p->name;
}

